@@ -51,6 +51,7 @@ class BaseNet(pl.LightningModule):
         n_classes : Optional[int],
         l1_strength : float = 0.0,
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **config : Any
     ) -> None:
         r"""
@@ -59,6 +60,10 @@ class BaseNet(pl.LightningModule):
             dims (int or tuple of int): The dimension of the data. e.g., 4, (8,8,1)
 
             n_classes (int, optional): Number of classes (labels) in the classification task. Do not specify this argument (set is as None) when you are performing regression tasks.
+                Default: ``None``
+
+            scheduler_config (``Dict``, optional): Dictionary specifying the LearningRate scheduler. It should include the class of the scheduler and its configurations. 
+                Default: {'scheduler' : None, 'scheduler_config' : None}
 
             **config (Any) : Hyperparameters that can be shared among different models. e.g., optimizer_type, lr, weight_decay, batch_size, bn, dropout_p, etc.
         """
@@ -73,10 +78,13 @@ class BaseNet(pl.LightningModule):
         self.n_classes = n_classes if n_classes is not None else None 
         self.config = config
         self.sample_weight = sample_weight
-
+        self.scheduler_config = scheduler_config
         config.update({'l1_strength' : l1_strength})
-        if 'scheduler' not in config.keys():
-            config.update({"scheduler" : OneCycleLR})
+
+        if list(self.scheduler_config.keys()) != ['scheduler', 'scheduler_config']:
+            raise ValueError(f"Wrong format of dictionary for scheduler_config. The keys for the argument should be 'scheduler' and 'scheduler_config'. Got {list(self.scheduler_config.keys())} instead.")
+            
+        
         self.save_hyperparameters(*config)
         
     def get_scheduler(self, optimizer, scheduler_class, **kwargs):
@@ -262,7 +270,7 @@ class BaseNet(pl.LightningModule):
             self.config.update({'scheduler' : None})
 
         scheduler_dict = {
-            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.config["scheduler"]),
+            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.scheduler_config["scheduler"], **(self.scheduler_config["scheduler_config"])),
             "interval" : "step"
         }
         return {"optimizer": optimizer, "lr_scheduler":scheduler_dict}
@@ -298,6 +306,7 @@ class ConvNet(pl.LightningModule):
         n_classes : Optional[int] = None,
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
         dataformats : Optional[str] = "NCHW",
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **config : Any
     )->None:
         r"""
@@ -317,7 +326,12 @@ class ConvNet(pl.LightningModule):
         self.sample_weight = sample_weight
         self.dataformats = dataformats
         self.config = config
+        self.scheduler_config = scheduler_config
         self.save_hyperparameters(*config)
+        
+        if list(self.scheduler_config.keys()) != ['scheduler', 'scheduler_config']:
+            raise ValueError(f"Wrong format of dictionary for scheduler_config. The keys for the argument should be 'scheduler' and 'scheduler_config'. Got {list(self.scheduler_config.keys())} instead.")
+
 
     def get_scheduler(self, optimizer, scheduler_class, **kwargs):
 
@@ -488,6 +502,7 @@ class ConvNet(pl.LightningModule):
 
     def configure_optimizers(self):
 
+        #optimizer_type_check(applicable_optimizer_type_list, self.config.optimizer_type)
         if self.hparams.optimizer_type == "adam":
             optimizer = torch.optim.Adam(self.parameters(),
                                      lr=self.hparams.lr,
@@ -502,7 +517,7 @@ class ConvNet(pl.LightningModule):
             self.config.update({'scheduler' : None})
 
         scheduler_dict = {
-            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.config["scheduler"]),
+            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.scheduler_config["scheduler"], **(self.scheduler_config["scheduler_config"])),
             "interval" : "step"
         }
         return {"optimizer": optimizer, "lr_scheduler":scheduler_dict}
@@ -531,6 +546,7 @@ class SegNet(pl.LightningModule):
         n_classes : Optional[int] = None,
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None,
         dataformats : Optional[str] = "NCHW",
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **config : Any
     )->None:
         r"""
@@ -549,6 +565,7 @@ class SegNet(pl.LightningModule):
         self.n_classes = n_classes
         self.sample_weight = sample_weight
         self.dataformats = dataformats
+        self.scheduler_config = scheduler_config
         self.config = config
         self.save_hyperparameters(*config)
         
@@ -715,6 +732,7 @@ class SegNet(pl.LightningModule):
 
     def configure_optimizers(self):
 
+        #optimizer_type_check(applicable_optimizer_type_list, self.config.optimizer_type)
         if self.hparams.optimizer_type == "adam":
             optimizer = torch.optim.Adam(self.parameters(),
                                      lr=self.hparams.lr,
@@ -729,7 +747,7 @@ class SegNet(pl.LightningModule):
             self.config.update({'scheduler' : None})
 
         scheduler_dict = {
-            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.config["scheduler"]),
+            "scheduler" : self.get_scheduler(optimizer=optimizer, scheduler_class=self.scheduler_config["scheduler"], **(self.scheduler_config["scheduler_config"])),
             "interval" : "step"
         }
         return {"optimizer": optimizer, "lr_scheduler":scheduler_dict}
