@@ -1,6 +1,7 @@
 from ast import Assert
 from pickle import BINBYTES
 from random import sample
+from sched import scheduler
 import torch
 import torch.nn as nn
 from astrape.utilities.utils_lightning import set_default_parameters
@@ -56,12 +57,12 @@ class _MLP(nn.Module):
         batch_size : int,
         optimizer_type : str,
         dropout_p : float,
-        scheduler,
         bn : bool,
         lr : float,
         weight_decay : float,
         l1_strength : float,
         momentum : float=0.9,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         activation_type : str = "relu", # "relu" or "leaky_relu"
         negative_slope : Optional[float] = None, # negative slope for leaky relu   
         sample_weight : Optional[Union["np.ndarray", "torch.Tensor"]] = None
@@ -85,7 +86,7 @@ class _MLP(nn.Module):
         self.dropout_p = dropout_p
         self.bn = bn
         self.lr = lr
-        self.scheduler = scheduler
+        self.scheduler_config = scheduler_config
         self.momentum = momentum
         self.weight_decay = weight_decay 
         self.sample_weight = sample_weight
@@ -167,6 +168,7 @@ class MLP(BaseNet):
         dims : Union[int, Tuple[int]],
         n_classes : int,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **hparams : Any
     ) -> None:
         r"""Multi-layer perceptron with identical # of hidden units.
@@ -219,7 +221,7 @@ class MLP(BaseNet):
             'l1_strength',
             'activation_type',
             'negative_slope',
-            'scheduler',
+            'scheduler_config',
             'momentum'
             ]
         default_parameter_dict = {
@@ -245,6 +247,7 @@ class MLP(BaseNet):
             dims=dims, 
             n_classes=n_classes, 
             sample_weight=sample_weight,
+            scheduler_config=scheduler_config,
             **hparams
         )
         # sanity check. Check if right arguments are declared.
@@ -279,7 +282,7 @@ class _ContractingMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        scheduler,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         activation_type : str = "relu",
         momentum : float=0.9,
         negative_slope : Optional[float] = None,
@@ -336,7 +339,7 @@ class _ContractingMLP(nn.Module):
         self.current_units = self.dims
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-        self.scheduler = scheduler
+        self.scheduler_config = scheduler_config
         self.momentum = momentum
 
         layers = []
@@ -475,6 +478,7 @@ class ContractingMLP(BaseNet):
         dims : Union[int, Tuple[int]],
         n_classes : Optional[int],
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        scheduler_config: Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **hparams : Any
     ) -> None:
         r"""Initialize ContractingMLP
@@ -521,7 +525,7 @@ class ContractingMLP(BaseNet):
             'l1_strength',
             'activation_type',
             'negative_slope',
-            'scheduler',
+            'scheduler_config',
             'momentum'
             ]
         default_parameter_dict = {
@@ -544,6 +548,7 @@ class ContractingMLP(BaseNet):
             dims=dims,
             n_classes=n_classes,
             sample_weight=sample_weight,
+            scheduler_config=scheduler_config,
             **hparams
         )
 
@@ -604,7 +609,7 @@ class _CustomMLP(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        scheduler,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         activation_type : str = "relu",
         negative_slope : Optional[float] = 0.2,
         n_hidden_units_list : Optional[List[int]] = None,
@@ -630,7 +635,7 @@ class _CustomMLP(nn.Module):
         self.l1_strength = l1_strength
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-        self.scheduler = scheduler
+        self.scheduler_config = scheduler_config
         self.momentum = momentum
 
         for n_hidden_units in self.n_hidden_units_list:
@@ -704,6 +709,7 @@ class CustomMLP(BaseNet):
         dims : Union[int, Tuple[int]],
         n_classes : int,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **hparams : Any
     ) -> None:
         
@@ -719,7 +725,7 @@ class CustomMLP(BaseNet):
             'weight_decay',
             'sample_weight',
             'l1_strength',
-            'scheduler',
+            'scheduler_config',
             'momentum'
             ]
         default_parameter_dict = {
@@ -747,6 +753,7 @@ class CustomMLP(BaseNet):
             dims=dims,
             n_classes=n_classes,
             sample_weight=sample_weight,
+            scheduler_config=scheduler_config,
             **hparams
         )
 
@@ -773,7 +780,7 @@ class _VGG(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        scheduler,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         activation_type : str = "relu",
         negative_slope : Optional[float] = None, 
         momentum : float=0.9,
@@ -807,7 +814,7 @@ class _VGG(nn.Module):
         self.l1_strength = l1_strength
         self.activation_type = activation_type
         self.negative_slope = negative_slope
-        self.scheduler = scheduler
+        self.scheduler_config = scheduler_config
         self.momentum = momentum
 
         self.avgpool = nn.AdaptiveAvgPool2d((7, 7))
@@ -896,6 +903,7 @@ class VGG(ConvNet):
         n_classes : int,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
         cfg : str ="VGG11",
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **hparams
     ) -> None:
         
@@ -914,7 +922,7 @@ class VGG(ConvNet):
             'l1_strength',
             'activation_type',
             'negative_slope',
-            'scheduler',
+            'scheduler_config',
             'momentum'
             ]
 
@@ -935,6 +943,7 @@ class VGG(ConvNet):
             sample_weight=sample_weight,
             model_type="VGG",
             cfg=cfg,
+            scheduler_config=scheduler_config,
             **hparams
         )
         # sanity check. Check if right arguments are declared.
@@ -962,7 +971,7 @@ class _ResNet18(nn.Module):
         lr : float,
         weight_decay : float,
         l1_strength : float,
-        scheduler,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         momentum : float=0.9,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None
     )->None:
@@ -981,7 +990,7 @@ class _ResNet18(nn.Module):
         self.l1_strength = l1_strength
         self.sample_weight = sample_weight
         self.momentum = momentum
-        self.scheduler = scheduler
+        self.scheduler_config = scheduler_config
 
         layers = [create_model()]
         if self.n_classes > 2:
@@ -999,6 +1008,7 @@ class ResNet18(ConvNet):
         dims : Union[int, Tuple[int]],
         n_classes : int,
         sample_weight : Optional[Union["torch.Tensor", "np.ndarray"]] = None,
+        scheduler_config : Optional[Dict[str, Any]] = {'scheduler' : None, 'scheduler_config' : None},
         **hparams
     )->None:
         self.dims = dims
@@ -1014,7 +1024,7 @@ class ResNet18(ConvNet):
             'weight_decay',
             'sample_weight',
             'l1_strength',
-            'scheduler',
+            'scheduler_config',
             'momentum'
             ]
         default_parameter_dict = {
@@ -1040,6 +1050,7 @@ class ResNet18(ConvNet):
             dims=dims,
             n_classes=n_classes,
             sample_weight=sample_weight,
+            schduler_config=scheduler_config,
             **hparams
         )
 
